unit uSearchEngineList;

interface

uses SysUtils, Classes, Contnrs, AdvCustomTreeView, AdvTreeViewData, AdvTreeView,

     uTCountryListItem, uWorldData, uTSearchEngineListItem, uSearchEngineData,
     uTSearchEngineProviderListItem,
     uSearchEngineGlobal;

type
 // TSearchEngineListViewMode
 TSearchEngineListViewMode = (selvmNone,selvmSearchEngineProvider,selvmCountries,selvmSubContinents,selvmContinents,selvmTradingOrgs);

 // TSearchEngineList
 TSearchEngineListObject = class
 strict private
  FSearchEngineListViewMode : TSearchEngineListViewMode;
 public
  WorldDataList: TObjectList;
  SearchEngineDataList: TObjectList;
  constructor Create;
  destructor  Destroy; override;
  // routines
  procedure DisplaySearchEnginesByProvider(tvView : TAdvTreeView);
  procedure DisplaySearchEnginesByCountry(tvView : TAdvTreeView);
  procedure DisplaySearchEnginesByContinents(tvView : TAdvTreeView);
  procedure DisplaySearchEnginesBySubContinents(tvView : TAdvTreeView);

  function  AddParentCountryNode(tvView : TAdvTreeView) : TAdvTreeViewNode;
  function  AddParentContinentNode(tvView : TAdvTreeView) : TAdvTreeViewNode;
  function  AddParentSubContinentNode(tvView : TAdvTreeView) : TAdvTreeViewNode;
  function  AddChildCountryNode(ParentNode : TAdvTreeViewNode; tvView : TAdvTreeView) : TAdvTreeViewNode;

  function  AddParentSearchEngineNode(tvView : TAdvTreeView) : TAdvTreeViewNode;
  function  AddChildSearchEngineNode(ParentNode : TAdvTreeViewNode; tvView : TAdvTreeView) : TAdvTreeViewNode;

  // properties
  property SearchEngineListViewMode : TSearchEngineListViewMode read FSearchEngineListViewMode write FSearchEngineListViewMode;

 end;

var
 SearchEngineListObject : TSearchEngineListObject;

implementation

{ TSearchEngineListObject }

function TSearchEngineListObject.AddChildCountryNode(ParentNode: TAdvTreeViewNode; tvView: TAdvTreeView): TAdvTreeViewNode;
begin
 result := nil;
 if tvView <> nil then
 begin
  result := tvView.AddNode(ParentNode);

 end;
end;

function TSearchEngineListObject.AddChildSearchEngineNode(ParentNode: TAdvTreeViewNode; tvView: TAdvTreeView): TAdvTreeViewNode;
begin
 result := nil;
 if tvView <> nil then
 begin
  result := tvView.AddNode(ParentNode);

 end;
end;

function TSearchEngineListObject.AddParentContinentNode(tvView: TAdvTreeView): TAdvTreeViewNode;
begin
 result := nil;
 if tvView <> nil then
 begin
  result := tvView.AddNode(nil);

 end;
end;

function TSearchEngineListObject.AddParentCountryNode(tvView: TAdvTreeView): TAdvTreeViewNode;
begin
 result := nil;
 if tvView <> nil then
 begin
  result := tvView.AddNode(nil);

 end;
end;

function TSearchEngineListObject.AddParentSearchEngineNode(tvView: TAdvTreeView): TAdvTreeViewNode;
begin
 result := nil;
 if tvView <> nil then
 begin
  result := tvView.AddNode(nil);

 end;
end;

function TSearchEngineListObject.AddParentSubContinentNode(tvView: TAdvTreeView): TAdvTreeViewNode;
begin
 result := nil;
 if tvView <> nil then
 begin
  result := tvView.AddNode(nil);

 end;
end;

constructor TSearchEngineListObject.Create;
begin
 inherited;
 FSearchEngineListViewMode := selvmNone;
 WorldDataList := TObjectList.Create(true);
 FillWorldDataList(WorldDataList);
 SearchEngineDataList := TObjectList.Create(true);
 FillSearchEngineList(SearchEngineDataList);
end;

destructor TSearchEngineListObject.Destroy;
begin

  inherited;
end;

procedure TSearchEngineListObject.DisplaySearchEnginesByContinents(tvView: TAdvTreeView);
var
 i,j: integer;
 ParentNode,ChildNode,SearchEngineNode: TAdvTreeViewNode;
begin
 // check if the treeview object is valid
 if tvView = nil then exit;
 if SearchEngineDataList.Count <= 0 then exit;
 // view mode
 FSearchEngineListViewMode := selvmContinents;
 with tvView do
 begin
  BeginUpdate;
  // clear
  Nodes.Clear;
  Groups.Clear;
  Columns.Clear;
  // options
  NodesAppearance.HeightMode := tnhmVariable;
  // columns
  Columns.Add.Text := '  ' + 'Country';
  Columns.Add.Text := '  ' + 'TLD';
  Columns.Add.Text := '  ' + 'URL';
  // nodes
  for i := 0 to WorldDataList.Count - 1 do
  begin
   // check if the item is a country
   if TCountryListItem(WorldDataList[i]).ItemType = ClitContinent then
   begin
    // continents
    ParentNode := AddParentContinentNode(tvView);
    ParentNode.Text[0] := TCountryListItem(WorldDataList[i]).ItemGermanRegionName;
    ParentNode.CheckTypes[0] := tvntCheckBox;
    ParentNode.DataObject := WorldDataList[i];
   end;
   if TCountryListItem(WorldDataList[i]).ItemType = clitCountry then
   begin
    // countries
    if ParentNode <> nil then
    begin
     ChildNode := AddChildCountryNode(ParentNode,tvView);
     ChildNode.Text[0] := TCountryListItem(WorldDataList[i]).ItemGermanRegionName;
     ChildNode.Text[1] := TCountryListItem(WorldDataList[i]).ItemTopLevelDomain;
     ChildNode.CheckTypes[0] := tvntCheckBox;
     ChildNode.DataObject := WorldDataList[i];
     // search engines
     for j := 0 to SearchEngineDataList.Count - 1 do
     begin
      if TCountryListItem(WorldDataList[i]).ItemCountryCode = TSearchEngineListItem(SearchEngineDataList[j]).ItemCountryCode then
      begin
       SearchEngineNode := AddChildSearchEngineNode(ChildNode,tvView);
       SearchEngineNode.Text[0] := TSearchEngineListItem(SearchEngineDataList[j]).ItemGermanName;
       SearchEngineNode.Text[1] := TSearchEngineListItem(SearchEngineDataList[j]).ItemSearchEngineURL;
       SearchEngineNode.CheckTypes[0] := tvntCheckBox;
       if TSearchEngineListItem(SearchEngineDataList[j]).ItemSelected then
        SearchEngineNode.Checked[0] := true
       else
        SearchEngineNode.Checked[0] := false;
       SearchEngineNode.DataObject := SearchEngineDataList[j];
      end;
     end;
    end;
   end;
  end;
  EndUpdate;
  ExpandAll;
 end;
end;

procedure TSearchEngineListObject.DisplaySearchEnginesByCountry(tvView: TAdvTreeView);
var
 i,j: integer;
 ParentNode,ChildNode,SearchEngineNode: TAdvTreeViewNode;
begin
 // check if the treeview object is valid
 if tvView = nil then exit;
 if SearchEngineDataList.Count <= 0 then exit;
 // view mode
 FSearchEngineListViewMode := selvmCountries;
 with tvView do
 begin
  BeginUpdate;
  // clear
  Nodes.Clear;
  Groups.Clear;
  Columns.Clear;
  // options
  NodesAppearance.HeightMode := tnhmVariable;
  // columns
  Columns.Add.Text := '  ' + 'Country';
  Columns.Add.Text := '  ' + 'TLD';
  Columns.Add.Text := '  ' + 'URL';
  // nodes
  for i := 0 to WorldDataList.Count - 1 do
  begin
   // check if the item is a country
   if TCountryListItem(WorldDataList[i]).ItemCountryCode <> -1 then
   begin
    ParentNode := AddParentCountryNode(tvView);
    ParentNode.Text[0] := TCountryListItem(WorldDataList[i]).ItemGermanRegionName;
    ParentNode.Text[1] := TCountryListItem(WorldDataList[i]).ItemTopLevelDomain;
    ParentNode.CheckTypes[0] := tvntCheckBox;
    ParentNode.DataObject := WorldDataList[i];
    // search engines
    for j := 0 to SearchEngineDataList.Count - 1 do
    begin
     if TCountryListItem(WorldDataList[i]).ItemCountryCode = TSearchEngineListItem(SearchEngineDataList[j]).ItemCountryCode then
     begin
      SearchEngineNode := AddChildSearchEngineNode(ParentNode,tvView);
      SearchEngineNode.Text[0] := TSearchEngineListItem(SearchEngineDataList[j]).ItemGermanName;
      SearchEngineNode.Text[1] := TSearchEngineListItem(SearchEngineDataList[j]).ItemSearchEngineURL;
      SearchEngineNode.CheckTypes[0] := tvntCheckBox;
      if TSearchEngineListItem(SearchEngineDataList[j]).ItemSelected then
       SearchEngineNode.Checked[0] := true
      else
       SearchEngineNode.Checked[0] := false;
      SearchEngineNode.DataObject := SearchEngineDataList[j];
     end;
    end;
   end;
  end;
  EndUpdate;
  ExpandAll;
 end;
end;

procedure TSearchEngineListObject.DisplaySearchEnginesByProvider(tvView: TAdvTreeView);
var
 i,j: integer;
 ParentNode,SearchEngineNode: TAdvTreeViewNode;
 CountryString: string;
 TempSearchEngineProviderListItem: TSearchEngineProviderListItem;
begin
 // check if the treeview object is valid
 if tvView = nil then exit;
 if SearchEngineDataList.Count <= 0 then exit;
 // view mode
 FSearchEngineListViewMode := selvmSearchEngineProvider;
 with tvView do
 begin
  BeginUpdate;
  // clear
  Nodes.Clear;
  Groups.Clear;
  Columns.Clear;
  // options
  NodesAppearance.HeightMode := tnhmVariable;
  // columns
  Columns.Add.Text := '  ' + 'Country';
  Columns.Add.Text := '  ' + 'TLD';
  Columns.Add.Text := '  ' + 'URL';
  // ###########################################################################
  // google
  // ###########################################################################
  ParentNode := SearchEngineListObject.AddParentSearchEngineNode(tvView);
  ParentNode.CheckTypes[0] := tvntCheckBox;
  TempSearchEngineProviderListItem := TSearchEngineProviderListItem.Create(sepGoogle);
  ParentNode.DataObject := TempSearchEngineProviderListItem;
  // search engines
  for i := 0 to SearchEngineDataList.Count - 1 do
  begin
   if TSearchEngineListItem(SearchEngineDataList[i]).ItemSearchEngineProvider = sepGoogle then
   begin
    CountryString := '';
    // find country name for the country
    for j := 0 to WorldDataList.Count - 1 do
    begin
     if TSearchEngineListItem(SearchEngineDataList[i]).ItemCountryCode = TCountryListItem(WorldDataList[j]).ItemCountryCode then
      CountryString := TCountryListItem(WorldDataList[j]).ItemGermanRegionName;
    end;
    SearchEngineNode := SearchEngineListObject.AddChildSearchEngineNode(ParentNode,tvView);
    SearchEngineNode.Text[0] := TSearchEngineListItem(SearchEngineDataList[i]).ItemGermanName;
    if CountryString <> '' then
     SearchEngineNode.Text[0] := SearchEngineNode.Text[0] + ' (' + CountryString + ')';
    SearchEngineNode.Text[1] := TSearchEngineListItem(SearchEngineDataList[i]).ItemSearchEngineURL;
    SearchEngineNode.CheckTypes[0] := tvntCheckBox;
    if TSearchEngineListItem(SearchEngineDataList[i]).ItemSelected then
     SearchEngineNode.Checked[0] := true
    else
     SearchEngineNode.Checked[0] := false;
    SearchEngineNode.DataObject := SearchEngineDataList[i];
   end;
  end;
  // ###########################################################################
  // yahoo
  // ###########################################################################



  // ###########################################################################
  //
  // ###########################################################################
  EndUpdate;
  ExpandAll;
 end;
end;

procedure TSearchEngineListObject.DisplaySearchEnginesBySubContinents(tvView: TAdvTreeView);
var
 i,j: integer;
 ParentNode,ChildNode,SearchEngineNode: TAdvTreeViewNode;
 ContinentString: string;
begin
 // check if the treeview object is valid
 if tvView = nil then exit;
 if SearchEngineDataList.Count <= 0 then exit;
 ContinentString := '';
 // view mode
 FSearchEngineListViewMode := selvmSubContinents;
 with tvView do
 begin
  BeginUpdate;
  // clear
  Nodes.Clear;
  Groups.Clear;
  Columns.Clear;
  // options
  NodesAppearance.HeightMode := tnhmVariable;
  // columns
  Columns.Add.Text := '  ' + 'Country';
  Columns.Add.Text := '  ' + 'TLD';
  Columns.Add.Text := '  ' + 'URL';
  // nodes
  for i := 0 to WorldDataList.Count - 1 do
  begin
   // check if the item is a country
   if TCountryListItem(WorldDataList[i]).ItemType = ClitContinent then
    // continents
    ContinentString := TCountryListItem(WorldDataList[i]).ItemGermanRegionName;
   // check if the item is a country
   if TCountryListItem(WorldDataList[i]).ItemType = ClitSubContinent then
   begin
    // subcontinents
    ParentNode := AddParentContinentNode(tvView);
    ParentNode.Text[0] := TCountryListItem(WorldDataList[i]).ItemGermanRegionName + ' (' + ContinentString + ')';
    ParentNode.CheckTypes[0] := tvntCheckBox;
    ParentNode.DataObject := WorldDataList[i];
   end;
   if TCountryListItem(WorldDataList[i]).ItemType = clitCountry then
   begin
    // countries
    if ParentNode <> nil then
    begin
     ChildNode := AddChildCountryNode(ParentNode,tvView);
     ChildNode.Text[0] := TCountryListItem(WorldDataList[i]).ItemGermanRegionName;
     ChildNode.Text[1] := TCountryListItem(WorldDataList[i]).ItemTopLevelDomain;
     ChildNode.CheckTypes[0] := tvntCheckBox;
     ChildNode.DataObject := WorldDataList[i];
     // search engines
     for j := 0 to SearchEngineDataList.Count - 1 do
     begin
      if TCountryListItem(WorldDataList[i]).ItemCountryCode = TSearchEngineListItem(SearchEngineDataList[j]).ItemCountryCode then
      begin
       SearchEngineNode := AddChildSearchEngineNode(ChildNode,tvView);
       SearchEngineNode.Text[0] := TSearchEngineListItem(SearchEngineDataList[j]).ItemGermanName;
       SearchEngineNode.Text[1] := TSearchEngineListItem(SearchEngineDataList[j]).ItemSearchEngineURL;
       SearchEngineNode.CheckTypes[0] := tvntCheckBox;
       if TSearchEngineListItem(SearchEngineDataList[j]).ItemSelected then
        SearchEngineNode.Checked[0] := true
       else
        SearchEngineNode.Checked[0] := false;
       SearchEngineNode.DataObject := SearchEngineDataList[j];
      end;
     end;
    end;
   end;
  end;
  EndUpdate;
  ExpandAll;
 end;
end;

initialization
 // SearchEngineListObject
 SearchEngineListObject := TSearchEngineListObject.Create;

finalization
 // SearchEngineListObject
 if SearchEngineListObject <> nil then
  FreeAndNil(SearchEngineListObject);

end.
